# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('biologicallyderivedproducts', '0001_initial'),
        ('devicedefinitions', '0001_initial'),
        ('devices', '0001_initial'),
        ('locations', '0001_initial'),
        ('medicationknowledges', '0001_initial'),
        ('medications', '0001_initial'),
        ('nutritionproducts', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='InventoryItemDescription',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('language', models.CharField(choices=[('ar', 'Arabic'), ('bg', 'Bulgarian'), ('bg-BG', 'Bulgarian (Bulgaria)'), ('bn', 'Bengali'), ('cs', 'Czech'), ('cs-CZ', 'Czech (Czechia)'), ('bs', 'Bosnian'), ('bs-BA', 'Bosnian (Bosnia and Herzegovina)'), ('da', 'Danish'), ('da-DK', 'Danish (Denmark)'), ('de', 'German'), ('de-AT', 'German (Austria)'), ('de-CH', 'German (Switzerland)'), ('de-DE', 'German (Germany)'), ('el', 'Greek'), ('el-GR', 'Greek (Greece)'), ('en', 'English'), ('en-AU', 'English (Australia)'), ('en-CA', 'English (Canada)'), ('en-GB', 'English (Great Britain)'), ('en-IN', 'English (India)'), ('en-NZ', 'English (New Zealand)'), ('en-SG', 'English (Singapore)'), ('en-US', 'English (United States)'), ('es', 'Spanish'), ('es-AR', 'Spanish (Argentina)'), ('es-ES', 'Spanish (Spain)'), ('es-UY', 'Spanish (Uruguay)'), ('et', 'Estonian'), ('et-EE', 'Estonian (Estonia)'), ('fi', 'Finnish'), ('fr', 'French'), ('fr-BE', 'French (Belgium)'), ('fr-CH', 'French (Switzerland)'), ('fr-FR', 'French (France)'), ('fi-FI', 'Finnish (Finland)'), ('fr-CA', 'French (Canada)'), ('fy', 'Frisian'), ('fy-NL', 'Frisian (Netherlands)'), ('hi', 'Hindi'), ('hr', 'Croatian'), ('hr-HR', 'Croatian (Croatia)'), ('is', 'Icelandic'), ('is-IS', 'Icelandic (Iceland)'), ('it', 'Italian'), ('it-CH', 'Italian (Switzerland)'), ('it-IT', 'Italian (Italy)'), ('ja', 'Japanese'), ('ko', 'Korean'), ('lt', 'Lithuanian'), ('lt-LT', 'Lithuanian (Lithuania)'), ('lv', 'Latvian'), ('lv-LV', 'Latvian (Latvia)'), ('nl', 'Dutch'), ('nl-BE', 'Dutch (Belgium)'), ('nl-NL', 'Dutch (Netherlands)'), ('no', 'Norwegian'), ('no-NO', 'Norwegian (Norway)'), ('pa', 'Punjabi'), ('pl', 'Polish'), ('pl-PL', 'Polish (Poland)'), ('pt', 'Portuguese'), ('pt-PT', 'Portuguese (Portugal)'), ('pt-BR', 'Portuguese (Brazil)'), ('ro', 'Romanian'), ('ro-RO', 'Romanian (Romania)'), ('ru', 'Russian'), ('ru-RU', 'Russian (Russia)'), ('sk', 'Slovakian'), ('sk-SK', 'Slovakian (Slovakia)'), ('sl', 'Slovenian'), ('sl-SI', 'Slovenian (Slovenia)'), ('sr', 'Serbian'), ('sr-RS', 'Serbian (Serbia)'), ('sv', 'Swedish'), ('sv-SE', 'Swedish (Sweden)'), ('te', 'Telugu'), ('zh', 'Chinese'), ('zh-CN', 'Chinese (China)'), ('zh-HK', 'Chinese (Hong Kong)'), ('zh-SG', 'Chinese (Singapore)'), ('zh-TW', 'Chinese (Taiwan)')], max_length=255, null=True)),
                ('description', models.TextField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='InventoryItemName',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('name_type', models.CharField(choices=[('trade_name', 'Trade Name'), ('alias', 'Alias'), ('original_name', 'Original Name'), ('preferred', 'Preferred')], max_length=255, null=True)),
                ('language', models.CharField(choices=[('ar', 'Arabic'), ('bg', 'Bulgarian'), ('bg-BG', 'Bulgarian (Bulgaria)'), ('bn', 'Bengali'), ('cs', 'Czech'), ('cs-CZ', 'Czech (Czechia)'), ('bs', 'Bosnian'), ('bs-BA', 'Bosnian (Bosnia and Herzegovina)'), ('da', 'Danish'), ('da-DK', 'Danish (Denmark)'), ('de', 'German'), ('de-AT', 'German (Austria)'), ('de-CH', 'German (Switzerland)'), ('de-DE', 'German (Germany)'), ('el', 'Greek'), ('el-GR', 'Greek (Greece)'), ('en', 'English'), ('en-AU', 'English (Australia)'), ('en-CA', 'English (Canada)'), ('en-GB', 'English (Great Britain)'), ('en-IN', 'English (India)'), ('en-NZ', 'English (New Zealand)'), ('en-SG', 'English (Singapore)'), ('en-US', 'English (United States)'), ('es', 'Spanish'), ('es-AR', 'Spanish (Argentina)'), ('es-ES', 'Spanish (Spain)'), ('es-UY', 'Spanish (Uruguay)'), ('et', 'Estonian'), ('et-EE', 'Estonian (Estonia)'), ('fi', 'Finnish'), ('fr', 'French'), ('fr-BE', 'French (Belgium)'), ('fr-CH', 'French (Switzerland)'), ('fr-FR', 'French (France)'), ('fi-FI', 'Finnish (Finland)'), ('fr-CA', 'French (Canada)'), ('fy', 'Frisian'), ('fy-NL', 'Frisian (Netherlands)'), ('hi', 'Hindi'), ('hr', 'Croatian'), ('hr-HR', 'Croatian (Croatia)'), ('is', 'Icelandic'), ('is-IS', 'Icelandic (Iceland)'), ('it', 'Italian'), ('it-CH', 'Italian (Switzerland)'), ('it-IT', 'Italian (Italy)'), ('ja', 'Japanese'), ('ko', 'Korean'), ('lt', 'Lithuanian'), ('lt-LT', 'Lithuanian (Lithuania)'), ('lv', 'Latvian'), ('lv-LV', 'Latvian (Latvia)'), ('nl', 'Dutch'), ('nl-BE', 'Dutch (Belgium)'), ('nl-NL', 'Dutch (Netherlands)'), ('no', 'Norwegian'), ('no-NO', 'Norwegian (Norway)'), ('pa', 'Punjabi'), ('pl', 'Polish'), ('pl-PL', 'Polish (Poland)'), ('pt', 'Portuguese'), ('pt-PT', 'Portuguese (Portugal)'), ('pt-BR', 'Portuguese (Brazil)'), ('ro', 'Romanian'), ('ro-RO', 'Romanian (Romania)'), ('ru', 'Russian'), ('ru-RU', 'Russian (Russia)'), ('sk', 'Slovakian'), ('sk-SK', 'Slovakian (Slovakia)'), ('sl', 'Slovenian'), ('sl-SI', 'Slovenian (Slovenia)'), ('sr', 'Serbian'), ('sr-RS', 'Serbian (Serbia)'), ('sv', 'Swedish'), ('sv-SE', 'Swedish (Sweden)'), ('te', 'Telugu'), ('zh', 'Chinese'), ('zh-CN', 'Chinese (China)'), ('zh-HK', 'Chinese (Hong Kong)'), ('zh-SG', 'Chinese (Singapore)'), ('zh-TW', 'Chinese (Taiwan)')], max_length=255, null=True)),
                ('name', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='InventoryItemProductReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='InventoryItemReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='InventoryItemResponsibleOrganization',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='InventoryItemAssociation',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('association_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='inventory_item_associations_association_types', to='base.codeableconcept')),
                ('quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='inventory_item_associations_quantity', to='base.ratio')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='InventoryItem',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('active', 'Active'), ('inactive', 'Inactive'), ('draft', 'Draft'), ('entered-in-error', 'Entered in Error'), ('unknown', 'Unknown')], max_length=255, null=True)),
                ('base_unit', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='inventory_items_base_units', to='base.codeableconcept')),
                ('category', models.ManyToManyField(blank=True, related_name='inventory_items_categories', to='base.codeableconcept')),
                ('code', models.ManyToManyField(blank=True, related_name='inventory_items_codes', to='base.codeableconcept')),
                ('identifier', models.ManyToManyField(blank=True, related_name='inventory_items_identifier', to='base.identifier')),
                ('inventory_status', models.ManyToManyField(blank=True, related_name='inventory_items_inventory_statuses', to='base.codeableconcept')),
                ('net_content', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='inventory_items_net_contents', to='base.quantity')),
                ('association', models.ManyToManyField(blank=True, related_name='inventory_items_associations', to='inventoryitems.inventoryitemassociation')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='InventoryItemAssociationTypeReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('biological_derived_product', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='association_type_reference_biological_derived_products', to='biologicallyderivedproducts.biologicallyderivedproduct')),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='association_type_reference_devices', to='devices.device')),
                ('device_definition', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='association_type_reference_device_definitions', to='devicedefinitions.devicedefinition')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='association_type_reference_identifiers', to='base.identifier')),
                ('inventor_item', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='association_type_reference_inventory_items', to='inventoryitems.inventoryitem')),
                ('medication', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='association_type_reference_medications', to='medications.medication')),
                ('medication_knowledge', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='association_type_reference_medication_knowledge', to='medicationknowledges.medicationknowledge')),
                ('nutrition_product', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='association_type_reference_nutrition_products', to='nutritionproducts.nutritionproduct')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='inventoryitemassociation',
            name='related_item',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='inventory_item_associations_related_items', to='inventoryitems.inventoryitemassociationtypereference'),
        ),
        migrations.CreateModel(
            name='InventoryItemCharacteristic',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('value_string', models.CharField(max_length=255, null=True)),
                ('value_integer', models.IntegerField(null=True)),
                ('value_decimal', models.DecimalField(decimal_places=2, max_digits=10, null=True)),
                ('value_boolean', models.BooleanField(null=True)),
                ('value_url', models.URLField(null=True)),
                ('value_date_time', models.DateTimeField(null=True)),
                ('characteristic_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='inventory_item_characteristics_characteristic_types', to='base.codeableconcept')),
                ('value_annotation', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='inventory_item_characteristics_value_annotations', to='base.annotation')),
                ('value_codeable_concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='inventory_item_characteristics_value_codeable_concepts', to='base.codeableconcept')),
                ('value_duration', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='inventory_item_characteristics_value_durations', to='base.quantity')),
                ('value_quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='inventory_item_characteristics_value_quantities', to='base.quantity')),
                ('value_range', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='inventory_item_characteristics_value_ranges', to='base.range')),
                ('value_ratio', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='inventory_item_characteristics_value_ratios', to='base.ratio')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='inventoryitem',
            name='characteristic',
            field=models.ManyToManyField(blank=True, related_name='inventory_items_characteristics', to='inventoryitems.inventoryitemcharacteristic'),
        ),
        migrations.AddField(
            model_name='inventoryitem',
            name='description',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='inventory_items_descriptions', to='inventoryitems.inventoryitemdescription'),
        ),
        migrations.CreateModel(
            name='InventoryItemInstance',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('lot_number', models.CharField(max_length=255, null=True)),
                ('expiry', models.DateTimeField(null=True)),
                ('identifier', models.ManyToManyField(blank=True, related_name='inventory_item_instances', to='base.identifier')),
                ('location', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='inventory_item_instances_locations', to='locations.locationreference')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0004_initial'),
        ('explanationofbenefits', '0002_initial'),
        ('patients', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='explanationofbenefit',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='explanation_of_benefits_patient', to='patients.patientreference'),
        ),
        migrations.AddField(
            model_name='explanationofbenefit',
            name='patient_paid',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='explanation_of_benefits_patient_paid', to='base.money'),
        ),
        migrations.AddField(
            model_name='explanationofbenefit',
            name='pre_auth_period',
            field=models.ManyToManyField(blank=True, related_name='explanation_of_benefits_pre_auth_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='explanationofbenefit',
            name='priority',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='explanation_of_benefits_priority', to='base.codeableconcept'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-27 15:57

import django.contrib.postgres.fields
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('base', '0005_initial'),
        ('careteams', '0004_initial'),
        ('devices', '0001_initial'),
        ('encounters', '0004_initial'),
        ('groups', '0003_initial'),
        ('healthcareservices', '0001_initial'),
        ('locations', '0001_initial'),
        ('patients', '0002_initial'),
        ('practitionerroles', '0002_initial'),
        ('practitioners', '0001_initial'),
        ('provenances', '0001_initial'),
        ('relatedpersons', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='provenance',
            name='activity',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_activity', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='provenance',
            name='authorization',
            field=models.ManyToManyField(blank=True, related_name='provenance_authorization', to='base.codeablereference'),
        ),
        migrations.AddField(
            model_name='provenance',
            name='based_on',
            field=models.ManyToManyField(blank=True, related_name='provenance_based_on', to='base.reference'),
        ),
        migrations.AddField(
            model_name='provenance',
            name='encounter',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_encounter', to='encounters.encounterreference'),
        ),
        migrations.AddField(
            model_name='provenance',
            name='location',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_location', to='locations.locationreference'),
        ),
        migrations.AddField(
            model_name='provenance',
            name='occurred_date_time',
            field=models.DateTimeField(null=True),
        ),
        migrations.AddField(
            model_name='provenance',
            name='occurred_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provinance_occurred_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='provenance',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_patient', to='patients.patientreference'),
        ),
        migrations.AddField(
            model_name='provenance',
            name='policy',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.URLField(), null=True, size=None),
        ),
        migrations.AddField(
            model_name='provenance',
            name='recorded',
            field=models.DateTimeField(null=True),
        ),
        migrations.AddField(
            model_name='provenance',
            name='signature',
            field=models.ManyToManyField(blank=True, related_name='provenance_signature', to='base.signature'),
        ),
        migrations.AddField(
            model_name='provenance',
            name='why',
            field=models.TextField(null=True),
        ),
        migrations.CreateModel(
            name='ProvenanceAgent',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('role', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_agent_role', to='base.codeableconcept')),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_agent_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='provenance',
            name='agent',
            field=models.ManyToManyField(blank=True, related_name='provenance_agent', to='provenances.provenanceagent'),
        ),
        migrations.CreateModel(
            name='ProvenanceAgentOnBehalfOf',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('care_team', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_agent_on_behalf_of_care_team', to='careteams.careteamreference')),
                ('group', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_agent_on_behalf_of_group', to='groups.groupreference')),
                ('healthcare_service', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_agent_on_behalf_of_healthcare_service', to='healthcareservices.healthcareservicereference')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_agent_on_behalf_of_identifier', to='base.identifier')),
                ('organization', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_agent_on_behalf_of_organization', to='base.organizationreference')),
                ('patient', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_agent_on_behalf_of_patient', to='patients.patientreference')),
                ('practitioner', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_agent_on_behalf_of_practitioner', to='practitioners.practitionerreference')),
                ('practitioner_role', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_agent_on_behalf_of_practitioner_role', to='practitionerroles.practitionerrolereference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='provenanceagent',
            name='on_behalf_of',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_agent_on_behalf_of', to='provenances.provenanceagentonbehalfof'),
        ),
        migrations.CreateModel(
            name='ProvenanceAgentWhoReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('care_team', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_agent_who_reference_care_team', to='careteams.careteamreference')),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_agent_who_reference_device', to='devices.devicereference')),
                ('group', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_agent_who_reference_group', to='groups.groupreference')),
                ('healthcare_service', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_agent_who_reference_healthcare_service', to='healthcareservices.healthcareservicereference')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_agent_who_reference_identifier', to='base.identifier')),
                ('organization', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_agent_who_reference_organization', to='base.organizationreference')),
                ('practitioner', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_agent_who_reference_practitioner', to='practitioners.practitionerreference')),
                ('practitioner_role', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_agent_who_reference_practitioner_role', to='practitionerroles.practitionerrolereference')),
                ('related_person', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_agent_who_reference_related_person', to='relatedpersons.relatedpersonreference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='provenanceagent',
            name='who',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_agent_who', to='provenances.provenanceagentwhoreference'),
        ),
        migrations.CreateModel(
            name='ProvenanceEntry',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('role', models.CharField(choices=[('revision', 'Revision'), ('quotation', 'Quotation'), ('source', 'Source'), ('instantiates', 'Instantiates'), ('removal', 'Removal')], default='revision', max_length=255, null=True)),
                ('agent', models.ManyToManyField(blank=True, related_name='provenance_entry_agent', to='provenances.provenanceagent')),
                ('what', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='provenance_entry_what', to='base.reference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='provenance',
            name='entity',
            field=models.ManyToManyField(blank=True, related_name='provenance_entity', to='provenances.provenanceentry'),
        ),
    ]



"""inventory items model choices."""

from django.db.models import TextChoices
from django.utils.translation import gettext_lazy as _


class InventoryItemNameTypeChoices(TextChoices):
    """inventory item name type choices."""

    TREADE_NAME = "trade_name", _("Trade Name")
    ALIAS = "alias", _("Alias")
    ORIGINAL_NAME = "original_name", _("Original Name")
    PREFERRED = "preferred", _("Preferred")


class InventoryItemNameLanguageChoices(TextChoices):
    """inventory item name language choices."""

    AR = "ar", _("Arabic")
    BG = "bg", _("Bulgarian")
    BG_BG = "bg-BG", _("Bulgarian (Bulgaria)")
    BN = "bn", _("Bengali")
    CS = "cs", _("Czech")
    CS_CZ = "cs-CZ", _("Czech (Czechia)")
    BS = "bs", _("Bosnian")
    BS_BA = "bs-BA", _("Bosnian (Bosnia and Herzegovina)")
    DA = "da", _("Danish")
    DA_DK = "da-DK", _("Danish (Denmark)")
    DE = "de", _("German")
    DE_AT = "de-AT", _("German (Austria)")
    DE_CH = "de-CH", _("German (Switzerland)")
    DE_DE = "de-DE", _("German (Germany)")
    EL = "el", _("Greek")
    EL_GR = "el-GR", _("Greek (Greece)")
    EN = "en", _("English")
    EN_AU = "en-AU", _("English (Australia)")
    EN_CA = "en-CA", _("English (Canada)")
    EN_GB = "en-GB", _("English (Great Britain)")
    EN_IN = "en-IN", _("English (India)")
    EN_NZ = "en-NZ", _("English (New Zealand)")
    EN_SG = "en-SG", _("English (Singapore)")
    EN_US = "en-US", _("English (United States)")
    ES = "es", _("Spanish")
    ES_AR = "es-AR", _("Spanish (Argentina)")
    ES_ES = "es-ES", _("Spanish (Spain)")
    ES_UY = "es-UY", _("Spanish (Uruguay)")
    ET = "et", _("Estonian")
    ET_EE = "et-EE", _("Estonian (Estonia)")
    FI = "fi", _("Finnish")
    FR = "fr", _("French")
    FR_BE = "fr-BE", _("French (Belgium)")
    FR_CH = "fr-CH", _("French (Switzerland)")
    FR_FR = "fr-FR", _("French (France)")
    FI_FI = "fi-FI", _("Finnish (Finland)")
    FR_CA = "fr-CA", _("French (Canada)")
    FY = "fy", _("Frisian")
    FY_NL = "fy-NL", _("Frisian (Netherlands)")
    HI = "hi", _("Hindi")
    HR = "hr", _("Croatian")
    HR_HR = "hr-HR", _("Croatian (Croatia)")
    IS = "is", _("Icelandic")
    IS_IS = "is-IS", _("Icelandic (Iceland)")
    IT = "it", _("Italian")
    IT_CH = "it-CH", _("Italian (Switzerland)")
    IT_IT = "it-IT", _("Italian (Italy)")
    JA = "ja", _("Japanese")
    KO = "ko", _("Korean")
    LT = "lt", _("Lithuanian")
    LT_LT = "lt-LT", _("Lithuanian (Lithuania)")
    LV = "lv", _("Latvian")
    LV_LV = "lv-LV", _("Latvian (Latvia)")
    NL = "nl", _("Dutch")
    NL_BE = "nl-BE", _("Dutch (Belgium)")
    NL_NL = "nl-NL", _("Dutch (Netherlands)")
    NO = "no", _("Norwegian")
    NO_NO = "no-NO", _("Norwegian (Norway)")
    PA = "pa", _("Punjabi")
    PL = "pl", _("Polish")
    PL_PL = "pl-PL", _("Polish (Poland)")
    PT = "pt", _("Portuguese")
    PT_PT = "pt-PT", _("Portuguese (Portugal)")
    PT_BR = "pt-BR", _("Portuguese (Brazil)")
    RO = "ro", _("Romanian")
    RO_RO = "ro-RO", _("Romanian (Romania)")
    RU = "ru", _("Russian")
    RU_RU = "ru-RU", _("Russian (Russia)")
    SK = "sk", _("Slovakian")
    SK_SK = "sk-SK", _("Slovakian (Slovakia)")
    SL = "sl", _("Slovenian")
    SL_SI = "sl-SI", _("Slovenian (Slovenia)")
    SR = "sr", _("Serbian")
    SR_RS = "sr-RS", _("Serbian (Serbia)")
    SV = "sv", _("Swedish")
    SV_SE = "sv-SE", _("Swedish (Sweden)")
    TE = "te", _("Telugu")  # codespell:ignore
    ZH = "zh", _("Chinese")
    ZH_CN = "zh-CN", _("Chinese (China)")
    ZH_HK = "zh-HK", _("Chinese (Hong Kong)")
    ZH_SG = "zh-SG", _("Chinese (Singapore)")
    ZH_TW = "zh-TW", _("Chinese (Taiwan)")


class InventoryItemStatusChoices(TextChoices):
    """inventory item status choices."""

    ACTIVE = "active", _("Active")
    INACTIVE = "inactive", _("Inactive")
    DRAFT = "draft", _("Draft")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")
    UNKNOWN = "unknown", _("Unknown")



"""Slots URL Configuration."""

from django.urls import path

from . import views

app_name = "slots"

urlpatterns = [
    path("slots/", views.SlotListView.as_view(), name="slot-list"),
    path("slots/<int:pk>/", views.SlotDetailView.as_view(), name="slot-detail"),
]



"""Nutrition Product."""

from django.urls import path

from . import views

app_name = "nutritionproducts"

urlpatterns = [
    path("nutritionproducts/", views.NutritionProductListView.as_view()),
    path("nutritionproducts/<int:pk>/", views.NutritionProductDetailView.as_view()),
]



"""Utility functions for dfhir."""

import re


def to_snake_case(text: str) -> str:
    """Convert a string to snake case.

    The words in the text are converted to lowercase and
    separated by underscores.

    Args:
        text: The string to convert.

    Returns:
        The snake case string.
    """
    s1 = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", text)
    return re.sub("([a-z0-9])([A-Z])", r"\1_\2", s1).lower().replace("-", "_")



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0002_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='DetectedIssueAuthorReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DetectedIssueCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DetectedIssueEvidence',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DetectedIssueMitigation',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('date', models.DateTimeField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DetectedIssueReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DetectedIssueSubjectReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DetectedIssue',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('preliminary', 'Preliminary'), ('final', 'Final'), ('entered-in-error', 'Entered in Error'), ('mitigated', 'Mitigated')], max_length=255, null=True)),
                ('severity', models.CharField(choices=[('high', 'High'), ('moderate', 'Moderate'), ('low', 'Low')], max_length=255, null=True)),
                ('identified_date_time', models.DateTimeField(null=True)),
                ('detail', models.TextField(null=True)),
                ('reference', models.URLField(null=True)),
                ('category', models.ManyToManyField(blank=True, related_name='detected_issue_category', to='base.codeableconcept')),
                ('code', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='detected_issue_code', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""Device Definition choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class DeviceDefinitionRegulatoryIdentifierType(models.TextChoices):
    """Device Definition Regulatory Identifier Type choices."""

    BASIC = "BASIC", _("basic")
    MASTER = "MASTER", _("master")
    LICENSE = "LICENSE", _("license")


class DeviceDefinitionCorrectiveActionScope(models.TextChoices):
    """Device Definition Corrective Action Scope choices."""

    MODEL = "model", _("model")
    LOT_NUMBERS = "lot_numbers", _("lot_numbers")
    SERIAL_NUMBERS = "serial_numbers", _("serial_numbers")



# Generated by Django 5.2 on 2025-04-23 12:46

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DeviceUsage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('active', 'Active'), ('completed', 'Completed'), ('not-done', 'Not Done'), ('entered-in-error', 'Entered in Error'), ('intended', 'Intended'), ('stopped', 'Stopped'), ('on-hold', 'On Hold')], default='active', max_length=255)),
                ('timing_date_time', models.DateTimeField(null=True)),
                ('date_asserted', models.DateTimeField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceUsageAdherence',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceUsageDerivedFromReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceUsageInformationSourceReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceUsageReasonCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceUsageReasonReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""Deviceassociations choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class DeviceAssociationStatus(models.TextChoices):
    """DeviceAssociationStatus choices."""

    IMPLANTED = "implanted", _("Implanted")
    EXPLANTED = "explanted", _("Explanted")
    ATTACHED = "attached", _("Attached")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")
    UNKNOWN = "unknown", _("Unknown")



"""Payment Notice URL configuration."""

from django.urls import path

from . import views

app_name = "paymentnotices"

urlpatterns = [
    path("paymentnotices/", views.PaymentNoticeListView.as_view()),
    path("paymentnotices/<int:pk>/", views.PaymentNoticeDetailView.as_view()),
]



"""genomic study serializers."""

from drf_writable_nested import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    ReferenceSerializer,
)
from dfhir.conditions.serializers import (
    ConditionObservationCodeableReferenceSerializer,
)
from dfhir.devices.serializers import DeviceReferenceSerializer
from dfhir.documentreferences.serializers import DocumentReferenceReferenceSerializer
from dfhir.encounters.serializers import EncounterReferenceSerializer
from dfhir.genomicstudies.models import (
    GenomicStudy,
    GenomicStudyAnalysis,
    GenomicStudyAnalysisDevice,
    GenomicStudyAnalysisInput,
    GenomicStudyAnalysisOutput,
    GenomicStudyAnalysisPerformer,
    GenomicStudyAnalysisPerformerActorReference,
    GenomicStudyAnalysisProtocolPerformedReference,
    GenomicStudyBasedOnReference,
    GenomicStudyReference,
    GenomicStudySubjectReference,
)
from dfhir.observations.serializers import (
    DocumentReferenceObservationReferenceSerializer,
)
from dfhir.practitioners.serializers import (
    PractitionerPractitionerRoleReferenceSerializer,
)
from dfhir.specimens.serializers import SpecimenReferenceSerializer


class GenomicStudyReferenceSerializer(BaseReferenceModelSerializer):
    """Genomic study reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = GenomicStudyReference
        exclude = ["created_at", "updated_at"]


class GenomicStudyAnalysisInputSerializer(WritableNestedModelSerializer):
    """Genomic study analysis input serializer."""

    file = DocumentReferenceReferenceSerializer(many=False, required=False)
    type = CodeableConceptSerializer(many=False, required=False)
    generated_by_identifier = IdentifierSerializer(many=False, required=False)
    generated_by_reference = GenomicStudyReferenceSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = GenomicStudyAnalysisInput
        exclude = ["created_at", "updated_at"]


class GenomicStudyAnalysisOutputSerializer(WritableNestedModelSerializer):
    """Genomic study analysis output serializer."""

    file = DocumentReferenceReferenceSerializer(many=False, required=False)
    type = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = GenomicStudyAnalysisOutput
        exclude = ["created_at", "updated_at"]


class GenomicStudyAnalysisPerformerActorReferenceSerializer(
    BaseReferenceModelSerializer
):
    """Genomic study analysis performer actor reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = GenomicStudyAnalysisPerformerActorReference
        exclude = ["created_at", "updated_at"]


class GenomicStudyAnalysisPerformerSerializer(WritableNestedModelSerializer):
    """Genomic study analysis performer serializer."""

    actor = GenomicStudyAnalysisPerformerActorReferenceSerializer(
        many=False, required=False
    )
    role = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = GenomicStudyAnalysisPerformer
        exclude = ["created_at", "updated_at"]


class GenomicStudyAnalysisDeviceSerializer(WritableNestedModelSerializer):
    """Genomic study analysis device serializer."""

    device = DeviceReferenceSerializer(many=False, required=False)
    function = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = GenomicStudyAnalysisDevice
        exclude = ["created_at", "updated_at"]


class GenomicStudyAnalysisProtocolPerformedReferenceSerializer(
    BaseReferenceModelSerializer
):
    """Genomic study analysis protocol performed serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = GenomicStudyAnalysisProtocolPerformedReference
        exclude = ["created_at", "updated_at"]


class GenomicStudyAnalysisSerializer(WritableNestedModelSerializer):
    """Genomic study analysis serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    method_type = CodeableConceptSerializer(many=True, required=False)
    change_type = CodeableConceptSerializer(many=True, required=False)
    genomic_build = CodeableConceptSerializer(many=False, required=False)
    focus = ReferenceSerializer(many=True, required=False)
    specimen = SpecimenReferenceSerializer(many=True, required=False)
    note = AnnotationSerializer(many=True, required=False)
    protocol_performed = GenomicStudyAnalysisProtocolPerformedReferenceSerializer(
        many=False, required=False
    )
    regions_studied = DocumentReferenceObservationReferenceSerializer(
        many=True, required=False
    )
    regions_called = DocumentReferenceObservationReferenceSerializer(
        many=True, required=False
    )
    input = GenomicStudyAnalysisInputSerializer(many=True, required=False)
    output = GenomicStudyAnalysisOutputSerializer(many=True, required=False)
    performer = GenomicStudyAnalysisPerformerSerializer(many=True, required=False)
    device = GenomicStudyAnalysisDeviceSerializer(many=True, required=False)

    class Meta:
        """meta options."""

        model = GenomicStudyAnalysis
        exclude = ["created_at", "updated_at"]


class GenomicStudySubjectReferenceSerializer(BaseReferenceModelSerializer):
    """Genomic study subject reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = GenomicStudySubjectReference
        exclude = ["created_at", "updated_at"]


class GenomicStudyBasedOnReferenceSerializer(BaseReferenceModelSerializer):
    """Genomic study based on reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = GenomicStudyBasedOnReference
        exclude = ["created_at", "updated_at"]


class GenomicStudySerializer(BaseWritableNestedModelSerializer):
    """Genomic study serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    type = CodeableConceptSerializer(many=True, required=False)
    subject = GenomicStudySubjectReferenceSerializer(many=False, required=False)
    encounter = EncounterReferenceSerializer(many=False, required=False)
    based_on = GenomicStudyBasedOnReferenceSerializer(many=True, required=False)
    referrer = PractitionerPractitionerRoleReferenceSerializer(
        many=False, required=False
    )
    interpreter = PractitionerPractitionerRoleReferenceSerializer(
        many=True, required=False
    )
    reason = ConditionObservationCodeableReferenceSerializer(many=True, required=False)
    note = AnnotationSerializer(many=True, required=False)
    analysis = GenomicStudyAnalysisSerializer(many=True, required=False)

    class Meta:
        """meta options."""

        model = GenomicStudy
        exclude = ["created_at", "updated_at"]



"""Appointment responses views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import AppointmentResponse
from .serializers import AppointmentResponseSerializer


class AppointmentResponseListView(APIView):
    """Appointment response list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: AppointmentResponseSerializer(many=True)})
    def get(self, request):
        """Get all appointment responses."""
        appointment_responses = AppointmentResponse.objects.all()
        serializer = AppointmentResponseSerializer(appointment_responses, many=True)
        return Response(serializer.data)

    @extend_schema(
        request=AppointmentResponseSerializer,
        responses={201: AppointmentResponseSerializer},
    )
    def post(self, request):
        """Create an appointment response."""
        serializer = AppointmentResponseSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class AppointmentResponseDetailView(APIView):
    """Appointment response detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get appointment response object."""
        try:
            return AppointmentResponse.objects.get(id=pk)
        except AppointmentResponse.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: AppointmentResponseSerializer})
    def get(self, request, pk=None):
        """Get appointment response detail."""
        appointment_response = self.get_object(pk)
        serializer = AppointmentResponseSerializer(appointment_response)
        return Response(serializer.data)

    @extend_schema(
        request=AppointmentResponseSerializer,
        responses={200: AppointmentResponseSerializer},
    )
    def patch(self, request, pk=None):
        """Update appointment response."""
        appointment_response = self.get_object(pk)
        serializer = AppointmentResponseSerializer(
            appointment_response, data=request.data
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)

    @extend_schema(responses={204: None})
    def delete(self, request, pk=None):
        """Delete an appointment response."""
        appointment_response = self.get_object(pk)
        appointment_response.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""Device Definitions views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import DeviceDefinition
from .serializers import DeviceDefinitionSerializer


class DeviceDefinitionListCreateView(APIView):
    """Device Definition list create view."""

    permission_classes = [AllowAny]

    @extend_schema(
        request=DeviceDefinitionSerializer, responses={201: DeviceDefinitionSerializer}
    )
    def post(self, request):
        """Create a device definition."""
        serializer = DeviceDefinitionSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)

    @extend_schema(responses={200: DeviceDefinitionSerializer(many=True)})
    def get(self, request):
        """Get all device definitions."""
        device_definitions = DeviceDefinition.objects.all()
        serializer = DeviceDefinitionSerializer(device_definitions, many=True)
        return Response(serializer.data)


class DeviceDefinitionDetailView(APIView):
    """Device Definition detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get device definition object."""
        try:
            return DeviceDefinition.objects.get(id=pk)
        except DeviceDefinition.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: DeviceDefinitionSerializer})
    def get(self, request, pk=None):
        """Get device definition detail."""
        device_definition = self.get_object(pk)
        serializer = DeviceDefinitionSerializer(device_definition)
        return Response(serializer.data)

    @extend_schema(
        request=DeviceDefinitionSerializer, responses={200: DeviceDefinitionSerializer}
    )
    def patch(self, request, pk=None):
        """Update device definition."""
        device_definition = self.get_object(pk)
        serializer = DeviceDefinitionSerializer(device_definition, data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)

    @extend_schema(responses={204: None})
    def delete(self, request, pk=None):
        """Delete a device definition."""
        device_definition = self.get_object(pk)
        device_definition.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""Practitioner filters."""

from django_filters import rest_framework as filters

from dfhir.practitioners.models import Practitioner


class PractitionerFilter(filters.FilterSet):
    """Practitioner filter."""

    user = filters.CharFilter(field_name="user")

    class Meta:
        """Meta class."""

        model = Practitioner
        fields = ["gender", "active"]



"""Coverage models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    Money,
    OrganizationReference,
    Period,
    SimpleQuantity,
    TimeStampedModel,
)

from . import choices


class CoveragePaymentByPartyReference(BaseReference):
    """Coverage By Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="coverage_by_reference_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        related_name="coverage_by_reference_patient",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        related_name="coverage_by_reference_related_person",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        related_name="coverage_by_reference_organization",
        null=True,
    )


class CoveragePaymentBy(TimeStampedModel):
    """Coverage Payment By model."""

    party = models.ForeignKey(
        CoveragePaymentByPartyReference,
        on_delete=models.DO_NOTHING,
        related_name="coverage_payment_by_party",
        null=True,
    )
    responsibility = models.CharField(
        max_length=255,
        null=True,
    )


class CoveragePolicyHolderReference(BaseReference):
    """Policy Holder Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="policy_holder_reference_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        related_name="policy_holder_reference_patient",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        related_name="policy_holder_reference_related_person",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        related_name="policy_holder_reference_organization",
        null=True,
    )


class CoverageClass(TimeStampedModel):
    """Coverage Class model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="coverage_class_type",
        null=True,
    )
    value = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="coverage_class_value",
        null=True,
    )
    name = models.CharField(max_length=255, null=True)


class CoverageCostToBeneficiaryException(TimeStampedModel):
    """Coverage Cost To Beneficiary Exception model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="coverage_cost_to_beneficiary_exception_type",
        null=True,
    )
    period = models.ForeignKey(
        Period,
        on_delete=models.DO_NOTHING,
        related_name="coverage_cost_to_beneficiary_exception_period",
        null=True,
    )


class CoverageCostToBeneficiary(TimeStampedModel):
    """Coverage Cost To Beneficiary model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="coverage_cost_to_beneficiary_type",
        null=True,
    )
    category = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="coverage_cost_to_beneficiary_category",
        null=True,
    )
    network = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="coverage_cost_to_beneficiary_network",
        null=True,
    )
    unit = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="coverage_cost_to_beneficiary_unit",
        null=True,
    )
    term = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="coverage_cost_to_beneficiary_term",
        null=True,
    )
    value_quantity = models.ForeignKey(
        SimpleQuantity,
        on_delete=models.DO_NOTHING,
        related_name="coverage_cost_to_beneficiary_value_quantity",
        null=True,
    )
    value_money = models.ForeignKey(
        Money,
        on_delete=models.DO_NOTHING,
        related_name="coverage_cost_to_beneficiary_value_money",
        null=True,
    )
    exception = models.ManyToManyField(
        CoverageCostToBeneficiaryException,
        related_name="coverage_cost_to_beneficiary_exception",
        blank=True,
    )


class Coverage(TimeStampedModel):
    """Coverage model."""

    identifier = models.ManyToManyField(
        Identifier,
        related_name="coverage_identifier",
        blank=True,
    )
    status = models.CharField(
        max_length=255, choices=choices.CoverageStatusChoices.choices
    )
    kind = models.CharField(max_length=255, choices=choices.CoverageKindChoices.choices)
    payment_by = models.ForeignKey(
        CoveragePaymentBy,
        on_delete=models.DO_NOTHING,
        related_name="coverage_payment_by",
        null=True,
    )
    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="coverage_type",
        null=True,
    )
    policy_holder = models.ForeignKey(
        CoveragePolicyHolderReference,
        on_delete=models.DO_NOTHING,
        related_name="coverage_policy_holder",
        null=True,
    )
    subscriber = models.ForeignKey(
        "patients.PatientRelatedPersonReference",
        on_delete=models.DO_NOTHING,
        related_name="coverage_subscriber",
        null=True,
    )
    subscriber_identifier = models.ManyToManyField(
        Identifier, related_name="coverage_subscriber_identifier", blank=True
    )
    beneficiary = models.ForeignKey(
        "patients.PatientReference",
        on_delete=models.DO_NOTHING,
        related_name="coverage_beneficiary",
        null=True,
    )
    dependent = models.CharField(max_length=255, null=True)
    relationship = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="coverage_relationship",
        null=True,
    )
    period = models.ForeignKey(
        Period,
        on_delete=models.DO_NOTHING,
        related_name="coverage_period",
        null=True,
    )
    insurer = models.ForeignKey(
        OrganizationReference,
        on_delete=models.DO_NOTHING,
        related_name="coverage_insurer",
        null=True,
    )
    klass = models.ManyToManyField(
        CoverageClass,
        related_name="coverage_class",
        blank=True,
    )
    order = models.IntegerField(null=True)
    network = models.CharField(max_length=255, null=True)
    cost_to_beneficiary = models.ManyToManyField(
        CoverageCostToBeneficiary,
        related_name="coverage_cost_to_beneficiary",
        blank=True,
    )
    subrogation = models.BooleanField(default=True)
    contract = models.ManyToManyField(
        "contracts.ContractReference",
        related_name="coverage_contract",
        blank=True,
    )
    # insurance_plan = models.ForeignKey(
    #     "insuranceplans.InsurancePlan",
    #     on_delete=models.DO_NOTHING,
    #     related_name="coverage_insurance_plan",
    #     null=True,
    # )


class CoverageReference(BaseReference):
    """Coverage reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="coverage_reference_identifier",
        null=True,
    )
    coverage = models.ForeignKey(
        Coverage,
        on_delete=models.DO_NOTHING,
        related_name="coverage_reference_coverage",
        null=True,
    )


class CoverageClaimResponseReference(BaseReference):
    """coverage claim response reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="coverage_claim_response_reference_identifier",
    )
    coverage = models.ForeignKey(
        Coverage,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="coverage_claim_response_reference_coverage",
    )
    claim_response = models.ForeignKey(
        "claimresponses.ClaimResponse",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="coverage_claim_response_reference_claim_response",
    )



"""medications views file."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.generics import ListAPIView
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.medications.models import (
    Medication,
    MedicationCodes,
    MedicationDoseForm,
    MedicationIngredientItem,
)
from dfhir.medications.serializers import (
    MedicationCodesSerializer,
    MedicationDoseFormSerializer,
    MedicationIngredientItemSerializer,
    MedicationSerializer,
)


class MedicationsList(APIView):
    """medications list API view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: MedicationSerializer(many=True)})
    def get(self, request):
        """Get medications list."""
        medications = Medication.objects.all()
        serializer = MedicationSerializer(medications, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(request=MedicationSerializer, responses={201: MedicationSerializer})
    def post(self, request):
        """Create medication."""
        serializer = MedicationSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class MedicationDetail(APIView):
    """medication detail API view."""

    def get_object(self, pk):
        """Get medication object."""
        try:
            return Medication.objects.get(pk=pk)
        except Medication.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: MedicationSerializer})
    def get(self, request, pk=None):
        """Get medication."""
        medication = self.get_object(pk)
        serializer = MedicationSerializer(medication)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={200: MedicationSerializer})
    def patch(self, request, pk=None):
        """Update medication."""
        medication = self.get_object(pk)
        serializer = MedicationSerializer(medication, data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    def delete(self, request, pk=None):
        """Delete medication."""
        medication = self.get_object(pk)
        medication.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)


@extend_schema(responses={200: MedicationSerializer(many=True)})
class MedicationCodesList(ListAPIView):
    """medication codes list API view."""

    queryset = MedicationCodes.objects.all()
    serializer_class = MedicationCodesSerializer
    permission_classes = [AllowAny]


@extend_schema(responses={200: MedicationSerializer(many=True)})
class MedicationDoseFormList(ListAPIView):
    """medication dose form list API view."""

    queryset = MedicationDoseForm.objects.all()
    serializer_class = MedicationDoseFormSerializer
    permission_classes = [AllowAny]


@extend_schema(responses={200: MedicationSerializer(many=True)})
class MedicationIngredientItemList(ListAPIView):
    """medication ingredient item list API view."""

    queryset = MedicationIngredientItem.objects.all()
    serializer_class = MedicationIngredientItemSerializer
    permission_classes = [AllowAny]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('allergyintolerances', '0003_initial'),
        ('base', '0003_initial'),
        ('patients', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='allergyintolerance',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='allergy_intolerance_patient', to='patients.patientreference'),
        ),
        migrations.AddField(
            model_name='allergyintolerance',
            name='type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='allergy_intolerance_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='allergyintolerance',
            name='verification_status',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='allergy_intolerance_verification_status', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='allergyintoleranceasserterreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='allergy_intolerance_asserter_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='allergyintoleranceasserterreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='allergy_intolerance_asserter_reference_patient', to='patients.patient'),
        ),
    ]



"""inventory item models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    TimeStampedModel,
)
from dfhir.base.models import Quantity as Duration
from dfhir.inventoryitems.choices import (
    InventoryItemNameLanguageChoices,
    InventoryItemNameTypeChoices,
    InventoryItemStatusChoices,
)
from dfhir.patients.models import PatientOrganizationReference


class InventoryItemAssociationTypeReference(BaseReference):
    """inventory item association type reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="association_type_reference_identifiers",
    )
    inventor_item = models.ForeignKey(
        "inventoryitems.InventoryItem",
        on_delete=models.SET_NULL,
        null=True,
        related_name="association_type_reference_inventory_items",
    )
    medication = models.ForeignKey(
        "medications.Medication",
        on_delete=models.SET_NULL,
        null=True,
        related_name="association_type_reference_medications",
    )
    medication_knowledge = models.ForeignKey(
        "medicationknowledges.MedicationKnowledge",
        on_delete=models.SET_NULL,
        null=True,
        related_name="association_type_reference_medication_knowledge",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.SET_NULL,
        null=True,
        related_name="association_type_reference_devices",
    )
    device_definition = models.ForeignKey(
        "devicedefinitions.DeviceDefinition",
        on_delete=models.SET_NULL,
        null=True,
        related_name="association_type_reference_device_definitions",
    )
    nutrition_product = models.ForeignKey(
        "nutritionproducts.NutritionProduct",
        on_delete=models.SET_NULL,
        null=True,
        related_name="association_type_reference_nutrition_products",
    )
    biological_derived_product = models.ForeignKey(
        "biologicallyderivedproducts.BiologicallyDerivedProduct",
        on_delete=models.SET_NULL,
        null=True,
        related_name="association_type_reference_biological_derived_products",
    )


class InventoryItemName(TimeStampedModel):
    """inventory item name model."""

    name_type = models.CharField(
        max_length=255, null=True, choices=InventoryItemNameTypeChoices.choices
    )
    language = models.CharField(
        max_length=255, null=True, choices=InventoryItemNameLanguageChoices.choices
    )
    name = models.CharField(max_length=255, null=True)


class InventoryItemResponsibleOrganization(TimeStampedModel):
    """inventory item responsible organization model."""

    code = models.ForeignKey(
        CodeableConcept,
        null=True,
        on_delete=models.CASCADE,
        related_name="inventory_item_responsible_organization_codes",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.SET_NULL,
        null=True,
        related_name="inventory_item_responsible_organization_organizations",
    )


class InventoryItemDescription(TimeStampedModel):
    """inventory item description model."""

    language = models.CharField(
        max_length=255, null=True, choices=InventoryItemNameLanguageChoices.choices
    )
    description = models.TextField(null=True)


class InventoryItemAssociation(TimeStampedModel):
    """inventory item association model."""

    association_type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="inventory_item_associations_association_types",
    )
    related_item = models.ForeignKey(
        InventoryItemAssociationTypeReference,
        on_delete=models.SET_NULL,
        null=True,
        related_name="inventory_item_associations_related_items",
    )
    quantity = models.ForeignKey(
        "base.Ratio",
        on_delete=models.SET_NULL,
        null=True,
        related_name="inventory_item_associations_quantity",
    )


class InventoryItemCharacteristic(TimeStampedModel):
    """inventory item characteristic model."""

    characteristic_type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        null=True,
        related_name="inventory_item_characteristics_characteristic_types",
    )
    value_string = models.CharField(max_length=255, null=True)
    value_integer = models.IntegerField(null=True)
    value_decimal = models.DecimalField(max_digits=10, decimal_places=2, null=True)
    value_boolean = models.BooleanField(null=True)
    value_url = models.URLField(null=True)
    value_date_time = models.DateTimeField(null=True)
    value_quantity = models.ForeignKey(
        "base.Quantity",
        on_delete=models.SET_NULL,
        null=True,
        related_name="inventory_item_characteristics_value_quantities",
    )
    value_range = models.ForeignKey(
        "base.Range",
        on_delete=models.SET_NULL,
        null=True,
        related_name="inventory_item_characteristics_value_ranges",
    )
    value_ratio = models.ForeignKey(
        "base.Ratio",
        on_delete=models.SET_NULL,
        null=True,
        related_name="inventory_item_characteristics_value_ratios",
    )
    value_annotation = models.ForeignKey(
        "base.Annotation",
        on_delete=models.SET_NULL,
        null=True,
        related_name="inventory_item_characteristics_value_annotations",
    )
    value_duration = models.ForeignKey(
        Duration,
        on_delete=models.SET_NULL,
        null=True,
        related_name="inventory_item_characteristics_value_durations",
    )
    value_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        null=True,
        related_name="inventory_item_characteristics_value_codeable_concepts",
    )


class InventoryItemInstance(TimeStampedModel):
    """inventory item instance model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="inventory_item_instances", blank=True
    )
    lot_number = models.CharField(max_length=255, null=True)
    expiry = models.DateTimeField(null=True)
    subject = models.ForeignKey(
        PatientOrganizationReference,
        on_delete=models.SET_NULL,
        null=True,
        related_name="inventory_item_instances_subjects",
    )
    location = models.ForeignKey(
        "locations.LocationReference",
        on_delete=models.SET_NULL,
        null=True,
        related_name="inventory_item_instances_locations",
    )


class InventoryItemProductReference(BaseReference):
    """inventory item product reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="inventory_item_product_reference_identifiers",
    )
    medication = models.ForeignKey(
        "medications.Medication",
        on_delete=models.SET_NULL,
        null=True,
        related_name="inventory_item_product_reference_medications",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.SET_NULL,
        null=True,
        related_name="inventory_item_product_reference_devices",
    )
    nutrition_product = models.ForeignKey(
        "nutritionproducts.NutritionProduct",
        on_delete=models.SET_NULL,
        null=True,
        related_name="inventory_item_product_references_nutrition_product",
    )
    biologically_derived_product = models.ForeignKey(
        "biologicallyderivedproducts.BiologicallyDerivedProduct",
        on_delete=models.SET_NULL,
        null=True,
        related_name="inventory_item_product_references_biologicallyderived_product",
    )


class InventoryItem(TimeStampedModel):
    """inventory item model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="inventory_items_identifier", blank=True
    )
    status = models.CharField(
        max_length=255, null=True, choices=InventoryItemStatusChoices.choices
    )
    category = models.ManyToManyField(
        CodeableConcept, related_name="inventory_items_categories", blank=True
    )
    code = models.ManyToManyField(
        CodeableConcept, related_name="inventory_items_codes", blank=True
    )
    name = models.ManyToManyField(
        InventoryItemName, related_name="inventory_items_names", blank=True
    )
    responsible_organization = models.ManyToManyField(
        InventoryItemResponsibleOrganization,
        related_name="inventory_items_responsible_organizations",
        blank=True,
    )
    description = models.ForeignKey(
        InventoryItemDescription,
        on_delete=models.SET_NULL,
        null=True,
        related_name="inventory_items_descriptions",
    )
    inventory_status = models.ManyToManyField(
        CodeableConcept, related_name="inventory_items_inventory_statuses", blank=True
    )
    base_unit = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        null=True,
        related_name="inventory_items_base_units",
    )
    net_content = models.ForeignKey(
        "base.Quantity",
        on_delete=models.SET_NULL,
        null=True,
        related_name="inventory_items_net_contents",
    )
    association = models.ManyToManyField(
        InventoryItemAssociation,
        related_name="inventory_items_associations",
        blank=True,
    )
    characteristic = models.ManyToManyField(
        InventoryItemCharacteristic,
        related_name="inventory_items_characteristics",
        blank=True,
    )
    instance = models.ForeignKey(
        InventoryItemInstance,
        null=True,
        on_delete=models.SET_NULL,
        related_name="inventory_items_insurances",
    )
    product_reference = models.ForeignKey(
        InventoryItemProductReference,
        on_delete=models.SET_NULL,
        null=True,
        related_name="inventory_items_product_references",
    )


class InventoryItemReference(BaseReference):
    """inventory item reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="inventory_item_references_identifiers",
    )
    inventory_item = models.ForeignKey(
        InventoryItem,
        on_delete=models.SET_NULL,
        null=True,
        related_name="inventory_item_references_inventory_items",
    )



"""Plandefinitions models."""

from django.db import models

from dfhir.base.models import (
    Age,
    BaseReference,
    CodeableConcept,
    Coding,
    ContactDetail,
    Duration,
    Expression,
    Identifier,
    Period,
    Quantity,
    Range,
    Ratio,
    RelatedArtifact,
    TimeStampedModel,
    Timing,
    TriggerDefinition,
    UsageContext,
)

from . import choices


class PlanDefinitionGoalTarget(TimeStampedModel):
    """Plan Definition Goal Target model."""

    measure = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_goal_target_measure",
        null=True,
    )
    detail_quantity = models.ForeignKey(
        Quantity,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_goal_target_detail_quantity",
        null=True,
    )
    detail_range = models.ForeignKey(
        Range,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_goal_target_detail_range",
        null=True,
    )
    detail_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_goal_target_detail_codeable_concept",
        null=True,
    )
    detail_string = models.CharField(
        max_length=255,
        null=True,
    )
    detail_boolean = models.BooleanField(
        null=True,
    )
    detail_integer = models.IntegerField(
        null=True,
    )
    detail_ratio = models.ForeignKey(
        Ratio,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_goal_target_detail_ratio",
        null=True,
    )

    due = models.ForeignKey(
        Period,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_goal_target_due",
        null=True,
    )


class PlanDefinitionGoal(TimeStampedModel):
    """Plan Definition Goal model."""

    category = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_goal_category",
        null=True,
    )
    description = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_goal_description",
        null=True,
    )
    priority = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_goal_priority",
        null=True,
    )
    start = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_goal_start",
        null=True,
    )
    addresses = models.ManyToManyField(
        CodeableConcept,
        related_name="plan_definition_goal_addresses",
    )
    documentation = models.ManyToManyField(
        RelatedArtifact,
        related_name="plan_definition_goal_documentation",
    )
    target = models.ManyToManyField(
        PlanDefinitionGoalTarget,
        related_name="plan_definition_goal_target",
    )


class PlanDefinitionActorOptionTypeReferenceReference(BaseReference):
    """Plan Definition Actor Option Type Reference Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="plan_definition_actor_option_type_reference_reference_identifier",
        null=True,
    )
    biologically_derived_product = models.ForeignKey(
        "biologicallyderivedproducts.BiologicallyDerivedProduct",
        on_delete=models.CASCADE,
        related_name="plan_definition_actor_option_type_reference_reference_biologically_derived_product",
        null=True,
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam",
        on_delete=models.CASCADE,
        related_name="plan_definition_actor_option_type_reference_reference_care_team",
        null=True,
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.CASCADE,
        related_name="plan_definition_actor_option_type_reference_reference_device",
        null=True,
    )
    device_definition = models.ForeignKey(
        "devicedefinitions.DeviceDefinition",
        on_delete=models.CASCADE,
        related_name="plan_definition_actor_option_type_reference_reference_device_definition",
        null=True,
    )
    endpoint = models.ForeignKey(
        "endpoints.Endpoint",
        on_delete=models.CASCADE,
        related_name="plan_definition_actor_option_type_reference_reference_endpoint",
        null=True,
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.CASCADE,
        related_name="plan_definition_actor_option_type_reference_reference_group",
        null=True,
    )
    healthcare_service = models.ForeignKey(
        "healthcareservices.HealthCareService",
        on_delete=models.CASCADE,
        related_name="plan_definition_actor_option_type_reference_reference_healthcare_service",
        null=True,
    )
    location = models.ForeignKey(
        "locations.Location",
        on_delete=models.CASCADE,
        related_name="plan_definition_actor_option_type_reference_reference_location",
        null=True,
    )
    medication = models.ForeignKey(
        "medications.Medication",
        on_delete=models.CASCADE,
        related_name="plan_definition_actor_option_type_reference_reference_medication",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.CASCADE,
        related_name="plan_definition_actor_option_type_reference_reference_organization",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="plan_definition_actor_option_type_reference_reference_patient",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="plan_definition_actor_option_type_reference_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="plan_definition_actor_option_type_reference_reference_practitioner_role",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        related_name="plan_definition_actor_option_type_reference_reference_related_person",
        null=True,
    )
    specimen = models.ForeignKey(
        "specimens.Specimen",
        on_delete=models.CASCADE,
        related_name="plan_definition_actor_option_type_reference_reference_specimen",
        null=True,
    )
    substance = models.ForeignKey(
        "substances.Substance",
        on_delete=models.CASCADE,
        related_name="plan_definition_actor_option_type_reference_reference_substance",
        null=True,
    )


class PlanDefintionActorOption(TimeStampedModel):
    """Plan Definition Actor Option model."""

    type = models.CharField(
        max_length=255,
        choices=choices.PlanDefinitionActionParticipantType.choices,
        default=choices.PlanDefinitionActionParticipantType.PATIENT,
    )
    # type_canonical = models.ForeignKey(
    #     CapabilityStatementCanonical,
    #     on_delete=models.DO_NOTHING,
    #     related_name="plan_definition_actor_option_type_canonical",
    #     null=True,
    # )
    type_reference = models.ForeignKey(
        PlanDefinitionActorOptionTypeReferenceReference,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_actor_option_type_reference",
        null=True,
    )
    role = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_actor_option_role",
        null=True,
    )


class PlanDefinitionActor(TimeStampedModel):
    """Plan Definition Actor model."""

    title = models.CharField(
        max_length=255,
        null=True,
    )
    description = models.CharField(
        max_length=255,
        null=True,
    )
    option = models.ManyToManyField(
        PlanDefintionActorOption,
        related_name="plan_definition_actor_option",
    )


class PlanDefinitionActionCondition(TimeStampedModel):
    """Plan Definition Action Condition model."""

    kind = models.CharField(
        max_length=255,
        choices=choices.PlanDefinitionActionConditionKind.choices,
        default=choices.PlanDefinitionActionConditionKind.APPLICABILITY,
    )
    expression = models.ForeignKey(
        Expression,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_action_condition_expression",
        null=True,
    )


class PlanDefinitionActionInput(TimeStampedModel):
    """Plan Definition Action Input model."""

    title = models.CharField(
        max_length=255,
        null=True,
    )
    # requirement = models.ForeignKey(
    #     DataRequirement,
    #     on_delete=models.DO_NOTHING,
    #     related_name="plan_definition_action_input_requirement",
    #     null=True,
    # )
    related_data = models.CharField(
        max_length=255,
        null=True,
    )


class PlanDefinitionActionOutput(TimeStampedModel):
    """Plan Definition Action Output model."""

    title = models.CharField(
        max_length=255,
        null=True,
    )
    # requirement = models.ForeignKey(
    #     DataRequirement,
    #     on_delete=models.DO_NOTHING,
    #     related_name="plan_definition_action_output_requirement",
    #     null=True,
    # )
    related_data = models.CharField(
        max_length=255,
        null=True,
    )


class PlanDefinitionActionRelatedAction(TimeStampedModel):
    """Plan Definition Action Related Action model."""

    target_id = models.CharField(
        max_length=255,
        null=True,
    )

    relationship = models.CharField(
        max_length=255,
        choices=choices.PlanDefinitionActionRelatedActionRelationship.choices,
        default=choices.PlanDefinitionActionRelatedActionRelationship.BEFORE_START,
    )
    end_relationship = models.CharField(
        max_length=255,
        choices=choices.PlanDefinitionActionRelatedActionRelationship.choices,
        default=choices.PlanDefinitionActionRelatedActionRelationship.BEFORE_START,
    )
    offset_duration = models.ForeignKey(
        Duration,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_action_related_action_offset_duration",
        null=True,
    )
    offset_range = models.ForeignKey(
        Range,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_action_related_action_offset_range",
        null=True,
    )


class PlanDefinitionActionParticipant(TimeStampedModel):
    """Plan Definition Action Participant model."""

    type = models.CharField(
        max_length=255,
        choices=choices.PlanDefinitionActionParticipantType.choices,
        default=choices.PlanDefinitionActionParticipantType.PATIENT,
    )
    # type_canonical = models.ForeignKey(
    #     CapabilityStatementCanonical,
    #     on_delete=models.DO_NOTHING,
    #     related_name="plan_definition_action_participant_type_canonical",
    #     null=True,
    # )
    type_reference = models.ForeignKey(
        PlanDefinitionActorOptionTypeReferenceReference,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_action_participant_type_reference",
        null=True,
    )
    role = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_action_participant_role",
        null=True,
    )
    function = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_action_participant_function",
        null=True,
    )


class PlanDefinitionActionDynamicValue(TimeStampedModel):
    """Plan Definition Action Dynamic Value model."""

    path = models.CharField(
        max_length=255,
        null=True,
    )
    expression = models.ForeignKey(
        Expression,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_action_dynamic_value_expression",
        null=True,
    )


class PlanDefinitionSubjectReference(BaseReference):
    """Plan Definition Subject Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="plan_definition_subject_reference_identifier",
        null=True,
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.CASCADE,
        related_name="plan_definition_subject_reference_group",
        null=True,
    )
    # medicinal_product_definition = models.ForeignKey(
    #     "medicinalproductdefinitions.MedicinalProductDefinition",
    #     on_delete=models.CASCADE,
    #     related_name="plan_definition_subject_reference_medicinal_product_definition",
    #     null=True,
    # )
    # substance_definition = models.ForeignKey(
    #     "substancedefinitions.SubstanceDefinition",
    #     on_delete=models.CASCADE,
    #     related_name="plan_definition_subject_reference_substance_definition",
    #     null=True,
    # )
    # adminstrable_product_definition = models.ForeignKey(
    #     "adminstrableproductdefinitions.AdminstrableProductDefinition",
    #     on_delete=models.CASCADE,
    #     related_name="plan_definition_subject_reference_adminstrable_product_definition",
    #     null=True,
    # )
    # manufactured_item_definition = models.ForeignKey(
    #     "manufactureditemdefinitions.ManufacturedItemDefinition",
    #     on_delete=models.CASCADE,
    #     related_name="plan_definition_subject_reference_manufactured_item_definition",
    #     null=True,
    # )
    # packaged_product_definition = models.ForeignKey(
    #     "packagedproductdefinitions.PackagedProductDefinition",
    #     on_delete=models.CASCADE,
    #     related_name="plan_definition_subject_reference_packaged_product_definition",
    #     null=True,
    # )


class PlanDefinitionAction(TimeStampedModel):
    """Plan Definition Action model."""

    link_id = models.CharField(
        max_length=255,
        null=True,
    )
    prefix = models.CharField(
        max_length=255,
        null=True,
    )
    title = models.CharField(
        max_length=255,
        null=True,
    )
    description = models.CharField(
        max_length=255,
        null=True,
    )
    text_equivalent = models.CharField(
        max_length=255,
        null=True,
    )
    priority = models.CharField(
        max_length=255,
        choices=choices.PlanDefinitionActionPriority.choices,
        default=choices.PlanDefinitionActionPriority.ROUTINE,
    )
    code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="plan_definition_action_code",
        null=True,
    )
    reason = models.ManyToManyField(
        CodeableConcept,
        related_name="plan_definition_action_reason",
        blank=True,
    )
    documentation = models.ManyToManyField(
        RelatedArtifact,
        related_name="plan_definition_action_documentation",
        blank=True,
    )
    goal_id = models.CharField(
        max_length=255,
        null=True,
    )
    subject_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_action_subject_codeable_concept",
        null=True,
    )
    subject_reference = models.ForeignKey(
        "groups.GroupReference",
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_action_subject_reference",
        null=True,
    )
    # subject_canonical = models.ForeignKey(
    #     Canonical,
    #     on_delete=models.DO_NOTHING,
    #     related_name="plan_definition_action_subject_canonical",
    #     null=True,
    # )
    trigger = models.ManyToManyField(
        TriggerDefinition,
        related_name="plan_definition_action_trigger",
        blank=True,
    )
    condition = models.ManyToManyField(
        PlanDefinitionActionCondition,
        related_name="plan_definition_action_condition",
    )
    input = models.ManyToManyField(
        PlanDefinitionActionInput,
        related_name="plan_definition_action_input",
        blank=True,
    )
    output = models.ManyToManyField(
        PlanDefinitionActionOutput,
        related_name="plan_definition_action_output",
        blank=True,
    )
    related_action = models.ManyToManyField(
        PlanDefinitionActionRelatedAction,
        related_name="plan_definition_action_related_action",
        blank=True,
    )
    timing_age = models.ForeignKey(
        Age,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_action_timing_age",
        null=True,
    )
    timing_duration = models.ForeignKey(
        Duration,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_action_timing_duration",
        null=True,
    )
    timing_range = models.ForeignKey(
        Range,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_action_timing_range",
        null=True,
    )
    timing_timing = models.ForeignKey(
        Timing,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_action_timing_timing",
        null=True,
    )
    location = models.ForeignKey(
        "locations.LocationCodeableReference",
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_action_location",
        null=True,
    )
    participant = models.ManyToManyField(
        PlanDefinitionActionParticipant,
        related_name="plan_definition_action_participant",
        blank=True,
    )
    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_action_type",
        null=True,
    )
    grouping_behavior = models.CharField(
        max_length=255,
        choices=choices.PlanDefinitionActionGroupingBehavior.choices,
        default=choices.PlanDefinitionActionGroupingBehavior.LOGICAL_GROUP,
    )
    selection_behavior = models.CharField(
        max_length=255,
        choices=choices.PlanDefinitionActionSelectionBehavior.choices,
        default=choices.PlanDefinitionActionSelectionBehavior.ALL,
    )
    required_behavior = models.CharField(
        max_length=255,
        choices=choices.PlanDefinitionActionRequiredBehavior.choices,
        default=choices.PlanDefinitionActionRequiredBehavior.MUST,
    )
    precheck_behavior = models.CharField(
        max_length=255,
        choices=choices.PlanDefinitionActionPrecheckBehavior.choices,
        default=choices.PlanDefinitionActionPrecheckBehavior.YES,
    )
    cardinality_behavior = models.CharField(
        max_length=255,
        choices=choices.PlanDefinitionActionCardinalityBehavior.choices,
        default=choices.PlanDefinitionActionCardinalityBehavior.SINGLE,
    )
    # definition_canonical = models.CharField(
    #     max_length=255,
    #     null=True,
    # )
    definition_uri = models.CharField(
        max_length=255,
        null=True,
    )
    # transform = models.ForeignKey(
    #     PlanDefinitionActionCanonicalMapStructure,
    #     on_delete=models.DO_NOTHING,
    #     related_name="plan_definition_action_transform",
    #     null=True,
    # )
    dynamic_value = models.ManyToManyField(
        PlanDefinitionActionDynamicValue,
        related_name="plan_definition_action_dynamic_value",
        blank=True,
    )
    action = models.ManyToManyField(
        "self",
        blank=True,
    )


class PlanDefinition(TimeStampedModel):
    """Plan Definition model."""

    url = models.URLField(
        null=True,
    )
    identifier = models.ManyToManyField(
        Identifier,
        related_name="plan_definition_identifier",
        blank=True,
    )
    version = models.CharField(
        max_length=255,
        null=True,
    )
    version_algorithm_string = models.CharField(
        max_length=255,
        null=True,
    )
    version_algorithm_coding = models.ForeignKey(
        Coding,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_version_algorithm_coding",
        null=True,
    )
    name = models.CharField(
        max_length=255,
        null=True,
    )
    title = models.CharField(
        max_length=255,
        null=True,
    )
    subtitle = models.CharField(
        max_length=255,
        null=True,
    )
    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_type",
        null=True,
    )
    status = models.CharField(
        max_length=255,
        choices=choices.PlanDefinitionStatus.choices,
        default=choices.PlanDefinitionStatus.ACTIVE,
    )
    experimental = models.BooleanField(
        default=False,
    )
    subject_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_subject_codeable_concept",
        null=True,
    )
    subject_reference = models.ForeignKey(
        PlanDefinitionSubjectReference,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_subject_reference",
        null=True,
    )
    # subject_canonical = models.ForeignKey(
    #     Canonical,
    #     on_delete=models.DO_NOTHING,
    #     related_name="plan_definition_subject_canonical",
    #     null=True,
    # )
    date = models.DateTimeField(
        null=True,
    )
    publisher = models.CharField(
        max_length=255,
        null=True,
    )
    contact = models.ManyToManyField(
        ContactDetail,
        related_name="plan_definition_contact",
        blank=True,
    )
    description = models.TextField(
        null=True,
    )
    use_context = models.ManyToManyField(
        UsageContext,
        related_name="plan_definition_use_context",
        blank=True,
    )
    jurisdiction = models.ManyToManyField(
        CodeableConcept,
        related_name="plan_definition_jurisdiction",
        blank=True,
    )
    purpose = models.TextField(
        null=True,
    )
    usage = models.TextField(
        null=True,
    )
    copyright = models.CharField(
        max_length=255,
        null=True,
    )
    copyright_label = models.CharField(
        max_length=255,
        null=True,
    )
    approval_date = models.DateField(
        null=True,
    )
    last_review_date = models.DateField(
        null=True,
    )
    effective_period = models.ForeignKey(
        Period,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_effective_period",
        null=True,
    )
    topic = models.ManyToManyField(
        CodeableConcept,
        related_name="plan_definition_topic",
        blank=True,
    )
    author = models.ManyToManyField(
        ContactDetail,
        related_name="plan_definition_author",
        blank=True,
    )
    editor = models.ManyToManyField(
        ContactDetail,
        related_name="plan_definition_editor",
        blank=True,
    )
    reviewer = models.ManyToManyField(
        ContactDetail,
        related_name="plan_definition_reviewer",
        blank=True,
    )
    endorser = models.ManyToManyField(
        ContactDetail,
        related_name="plan_definition_endorser",
        blank=True,
    )
    related_artifact = models.ManyToManyField(
        RelatedArtifact,
        related_name="plan_definition_related_artifact",
        blank=True,
    )
    # library = models.ManyToManyField(
    #     CanonicalLibrary,
    #     related_name="plan_definition_library",
    #     blank=True,
    # )
    goal = models.ManyToManyField(
        PlanDefinitionGoal,
        related_name="plan_definition_goal",
        blank=True,
    )
    actor = models.ManyToManyField(
        PlanDefinitionActor,
        related_name="plan_definition_actor",
        blank=True,
    )
    action = models.ManyToManyField(
        PlanDefinitionAction,
        related_name="plan_definition_action",
        blank=True,
    )
    as_needed_boolean = models.BooleanField(
        null=True,
    )
    as_needed_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="plan_definition_as_needed_codeable_concept",
        null=True,
    )



"""Serializers for the communications app."""

from drf_writable_nested import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AnnotationSerializer,
    AttachmentSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    CodeableReferenceSerializer,
    CodingSerializer,
    IdentifierSerializer,
    ReferenceSerializer,
)
from dfhir.communications.models import (
    Communication,
    CommunicationBsedOnReference,
    CommunicationPayload,
    CommunicationRecipientReference,
    CommunicationReference,
    CommunicationRequestReference,
    CommunicationSenderReference,
)
from dfhir.encounters.serializers import EncounterReferenceSerializer
from dfhir.patients.serializers import PatientGroupReferenceSerializer


class CommunicationRequestReferenceSerializer(BaseReferenceModelSerializer):
    """communication request reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta options."""

        model = CommunicationRequestReference
        exclude = ["created_at", "updated_at"]


class CommunicationRecipientReferenceSerializer(BaseReferenceModelSerializer):
    """communication recipient reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta options."""

        model = CommunicationRecipientReference
        exclude = ["created_at", "updated_at"]


class CommunicationPayloadSerializer(WritableNestedModelSerializer):
    """communication payload serializer."""

    content_attachment = AttachmentSerializer(many=False, required=False)
    content_reference = ReferenceSerializer(many=False, required=False)
    content_codeable_reference = CodeableReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta options."""

        model = CommunicationPayload
        exclude = ["created_at", "updated_at"]


class CommunicationSenderReferenceSerializer(BaseReferenceModelSerializer):
    """communication sender reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta options."""

        model = CommunicationSenderReference
        exclude = ["created_at", "updated_at"]


class CommunicationBsedOnReferenceSerializer(BaseReferenceModelSerializer):
    """communication based on reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta options."""

        model = CommunicationBsedOnReference
        exclude = ["created_at", "updated_at"]


class CommunicationReferenceSerializer(BaseReferenceModelSerializer):
    """communication reference serializer."""

    class Meta:
        """Meta options."""

        model = CommunicationReference
        exclude = ["created_at", "updated_at"]


class CommunicationSerializer(BaseWritableNestedModelSerializer):
    """communication serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    based_on = CommunicationBsedOnReferenceSerializer(many=True, required=False)
    part_of = ReferenceSerializer(many=True, required=False)
    in_response_to = CommunicationReferenceSerializer(many=True, required=False)
    status = CodingSerializer(many=False, required=False)
    status_reason = CodeableConceptSerializer(many=False, required=False)
    category = CodeableConceptSerializer(many=True, required=False)
    medium = CodeableConceptSerializer(many=True, required=False)
    subject = PatientGroupReferenceSerializer(many=False, required=False)
    topic = CodeableConceptSerializer(many=False, required=False)
    about = ReferenceSerializer(many=True, required=False)
    encounter = EncounterReferenceSerializer(many=False, required=False)
    recipient = CommunicationRecipientReferenceSerializer(many=True, required=False)
    sender = CommunicationSenderReferenceSerializer(many=False, required=False)
    reason = CodeableConceptSerializer(many=True, required=False)
    payload = CommunicationPayloadSerializer(many=True, required=False)
    note = AnnotationSerializer(many=True, required=False)

    class Meta:
        """meta options."""

        model = Communication
        exclude = ["created_at", "updated_at"]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='MolecularSequenceConcatenatedSequenceElement',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('ordinate_index', models.IntegerField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MolecularSequenceLiteral',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sequence_value', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MolecularSequenceConcatenated',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sequence_element', models.ManyToManyField(blank=True, related_name='molecular_sequence_concatenated_sequence_element', to='molecularsequences.molecularsequenceconcatenatedsequenceelement')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MolecularSequenceExtracted',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('start', models.IntegerField(null=True)),
                ('end', models.IntegerField(null=True)),
                ('reverse_complement', models.BooleanField(default=False)),
                ('coordinated_system', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='molecular_sequence_extracted_coordinated_system', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MolecularSequence',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('type', models.CharField(choices=[('aa', 'Amino acid'), ('dna', 'DNA'), ('rna', 'RNA')], max_length=255, null=True)),
                ('file', models.ManyToManyField(blank=True, related_name='molecular_sequence_file', to='base.attachment')),
                ('identifier', models.ManyToManyField(blank=True, related_name='molecular_sequence_identifier', to='base.identifier')),
                ('concatenated', models.ManyToManyField(blank=True, related_name='molecular_sequence_concatenated', to='molecularsequences.molecularsequenceconcatenated')),
                ('extracted', models.ManyToManyField(blank=True, related_name='molecular_sequence_extracted', to='molecularsequences.molecularsequenceextracted')),
                ('literal', models.ManyToManyField(blank=True, related_name='molecular_sequence_literal', to='molecularsequences.molecularsequenceliteral')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MolecularSequenceReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='molecular_sequence_reference_identifier', to='base.identifier')),
                ('molecular_sequence', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='molecular_sequence_reference_molecular_sequence', to='molecularsequences.molecularsequence')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='molecularsequenceextracted',
            name='starting_sequence',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='molecular_sequence_extracted_starting_sequence', to='molecularsequences.molecularsequencereference'),
        ),
        migrations.AddField(
            model_name='molecularsequenceconcatenatedsequenceelement',
            name='sequence',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='molecular_sequence_concatenated_sequence', to='molecularsequences.molecularsequencereference'),
        ),
        migrations.CreateModel(
            name='MolecularSequenceRelative',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('starting_sequence', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='molecular_sequence_relative_starting_sequence', to='molecularsequences.molecularsequencereference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='molecularsequence',
            name='relative',
            field=models.ManyToManyField(blank=True, related_name='molecular_sequence_relative', to='molecularsequences.molecularsequencerelative'),
        ),
        migrations.CreateModel(
            name='MolecularSequenceRelativeEdit',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('edit_order', models.IntegerField()),
                ('start', models.IntegerField(null=True)),
                ('end', models.IntegerField(null=True)),
                ('coordinate_system', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='molecular_sequence_relative_edit_coordinate_system', to='base.codeableconcept')),
                ('replaced_sequence', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='molecular_sequence_relative_edit_replaced_sequence', to='molecularsequences.molecularsequencereference')),
                ('replacement_sequence', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='molecular_sequence_relative_edit_replacement_sequence', to='molecularsequences.molecularsequencereference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='molecularsequencerelative',
            name='edit',
            field=models.ManyToManyField(blank=True, related_name='molecular_sequence_relative_edit', to='molecularsequences.molecularsequencerelativeedit'),
        ),
        migrations.CreateModel(
            name='MolecularSequenceRepeated',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('copy_count', models.IntegerField(null=True)),
                ('sequence_motif', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='molecular_sequence_repeated_sequence_motif', to='molecularsequences.molecularsequencereference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='molecularsequence',
            name='repeated',
            field=models.ManyToManyField(blank=True, related_name='molecular_sequence_repeated', to='molecularsequences.molecularsequencerepeated'),
        ),
    ]



"""Vision prescription serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    SimpleQuantitySerializer,
)
from dfhir.encounters.serializers import EncounterReferenceSerializer
from dfhir.patients.serializers import PatientReferenceSerializer
from dfhir.practitioners.serializers import (
    PractitionerPractitionerRoleReferenceSerializer,
)

from .models import (
    VisionPrescription,
    VisionPrescriptionBasedOnReference,
    VisionPrescriptionLensSpecification,
    VisionPrescriptionLensSpecificationPrism,
)


class VisionPrescriptionBasedOnReferenceSerializer(BaseReferenceModelSerializer):
    """Vision Prescription Based On Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = VisionPrescriptionBasedOnReference
        exclude = ["created_at", "updated_at"]


class VisionPrescriptionLensSpecificationPrismSerializer(WritableNestedModelSerializer):
    """Vision Prescription Lens Specification Prism Serializer."""

    class Meta:
        """Meta class."""

        model = VisionPrescriptionLensSpecificationPrism
        exclude = ["created_at", "updated_at"]


class VisionPrescriptionLensSpecificationSerializer(WritableNestedModelSerializer):
    """Vision Prescription Lens Specification Serializer."""

    product = CodeableConceptSerializer(many=False, required=False)
    prism = VisionPrescriptionLensSpecificationPrismSerializer(
        many=True, required=False
    )
    duration = SimpleQuantitySerializer(many=False, required=False)
    note = AnnotationSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = VisionPrescriptionLensSpecification
        exclude = ["created_at", "updated_at"]


class VisionPrescriptionSerializer(BaseWritableNestedModelSerializer):
    """Vision Prescription Serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    based_on = VisionPrescriptionBasedOnReferenceSerializer(many=True, required=False)
    patient = PatientReferenceSerializer(many=False, required=False)
    encounter = EncounterReferenceSerializer(many=False, required=False)
    prescriber = PractitionerPractitionerRoleReferenceSerializer(
        many=False, required=False
    )
    lens_specification = VisionPrescriptionLensSpecificationSerializer(
        many=True, required=False
    )

    class Meta:
        """Meta class."""

        model = VisionPrescription
        exclude = ["created_at", "updated_at"]



"""Explanation of Benefits Choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class ExplanationOfBenefitStatus(models.TextChoices):
    """Explanation of Benefit Status Choices."""

    ACTIVE = "active", _("Active")
    CANCELLED = "cancelled", _("Cancelled")
    DRAFT = "draft", _("Draft")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")
    UNKNOWN = "unknown", _("Unknown")


class ExplanationOfBenefitUse(models.TextChoices):
    """Explanation of Benefit Use Choices."""

    CLAIM = "claim", _("Claim")
    PREAUTHORIZATION = "preauthorization", _("Preauthorization")
    PREDETERMINATION = "predetermination", _("Predetermination")


class ExplanationOfBenefitOutcome(models.TextChoices):
    """Explanation of Benefit Outcome Choices."""

    QUEUED = "queued", _("Queued")
    COMPLETE = "complete", _("Complete")
    ERROR = "error", _("Error")
    PARTIAL = "partial", _("Partial")



"""Organizations URL Configuration."""

from django.urls import path

from . import views

app_name = "organizations"

urlpatterns = [
    path("organizations/", views.OrganizationListView.as_view()),
    path("organizations/<int:pk>/", views.OrganizationDetailView.as_view()),
]



"""risk assessment serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    PeriodSerializer,
    RangeSerializer,
    ReferenceSerializer,
)
from dfhir.conditions.serializers import ConditionReferenceSerializer
from dfhir.encounters.serializers import EncounterReferenceSerializer
from dfhir.patients.serializers import PatientGroupReferenceSerializer
from dfhir.riskassessments.models import (
    RiskAssessment,
    RiskAssessmentPerformerReference,
    RiskAssessmentPrediction,
    RiskAssessmentReasonCodeableConcept,
    RiskAssessmentReasonReference,
)


class RiskAssessmentPredictionSerializer(WritableNestedModelSerializer):
    """risk assessment prediction serializer."""

    outcome = CodeableConceptSerializer(required=False)
    probability_range = RangeSerializer(required=False)
    qualitative_risk = CodeableConceptSerializer(required=False)
    when_period = PeriodSerializer(required=False)
    when_range = RangeSerializer(required=False)

    class Meta:
        """Meta."""

        model = RiskAssessmentPrediction
        exclude = ["created_at", "updated_at"]


class RiskAssessmentPerformerReferenceSerializer(WritableNestedModelSerializer):
    """risk assessment performer reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta."""

        model = RiskAssessmentPerformerReference
        exclude = ["created_at", "updated_at"]


class RiskAssessmentReasonReferenceSerializer(BaseReferenceModelSerializer):
    """risk assessment reason reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta."""

        model = RiskAssessmentReasonReference
        exclude = ["created_at", "updated_at"]


class RiskAssessmentReasonCodeableReferenceSerializer(WritableNestedModelSerializer):
    """risk assessment reason codeable concept serializer."""

    concept = CodeableConceptSerializer(required=False)
    reference = RiskAssessmentReasonReferenceSerializer(required=False)

    class Meta:
        """Meta."""

        model = RiskAssessmentReasonCodeableConcept
        exclude = ["created_at", "updated_at"]


class RiskAssessmentSerializer(BaseWritableNestedModelSerializer):
    """risk assessment serializer."""

    identifier = IdentifierSerializer(required=False, many=True)
    based_on = ReferenceSerializer(required=False)
    parent = ReferenceSerializer(required=False)
    method = CodeableConceptSerializer(required=False)
    code = CodeableConceptSerializer(required=False)
    subject = PatientGroupReferenceSerializer(required=False)
    encounter = EncounterReferenceSerializer(required=False)
    occurrence_period = PeriodSerializer(required=False)
    condition = ConditionReferenceSerializer(required=False)
    performer = RiskAssessmentPerformerReferenceSerializer(required=False)
    reason = RiskAssessmentReasonCodeableReferenceSerializer(required=False)
    basis = ReferenceSerializer(required=False, many=True)
    prediction = RiskAssessmentPredictionSerializer(required=False, many=True)
    note = AnnotationSerializer(required=False, many=True)

    class Meta:
        """Meta options."""

        model = RiskAssessment
        exclude = ["created_at", "updated_at"]



"""Enrollment request serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    BaseReferenceModelSerializer,
    IdentifierSerializer,
    OrganizationReferenceSerializer,
)
from dfhir.coverages.serializers import CoverageReferenceSerializer
from dfhir.patients.serializers import PatientReferenceSerializer

from .models import (
    EnrollmentRequest,
    EnrollmentRequestProviderReference,
    EnrollmentRequestsReference,
)


class EnrollmentRequestProviderReferenceSerializer(BaseReferenceModelSerializer):
    """Enrollment Request Provider Reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = EnrollmentRequestProviderReference
        exclude = ["created_at", "updated_at"]


class EnrollmentRequestSerializer(WritableNestedModelSerializer):
    """Enrollment Request serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    insurer = OrganizationReferenceSerializer(many=False, required=False)
    provider = EnrollmentRequestProviderReferenceSerializer(many=False, required=False)
    candidate = PatientReferenceSerializer(many=False, required=False)
    coverage = CoverageReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = EnrollmentRequest
        exclude = ["created_at", "updated_at"]


class EnrollmentRequestReferenceSerializer(BaseReferenceModelSerializer):
    """Enrollment Requests Reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = EnrollmentRequestsReference
        exclude = ["created_at", "updated_at"]



"""Appointment Responses Serializers."""

from dfhir.appointments.serializers import AppointmentReferenceSerializer
from dfhir.base.serializers import (
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
)

from .models import (
    AppointmentResponse,
    AppointmentResponseActorReference,
)


class AppointmentResponseActorReferenceSerializer(BaseReferenceModelSerializer):
    """Appointment response actor reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = AppointmentResponseActorReference
        exclude = ["created_at", "updated_at"]


class AppointmentResponseSerializer(BaseWritableNestedModelSerializer):
    """Appointment response serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    appointment = AppointmentReferenceSerializer(many=False, required=False)
    participant_type = CodeableConceptSerializer(many=True, required=False)
    actor = AppointmentResponseActorReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = AppointmentResponse
        exclude = ["created_at", "updated_at"]



"""Related Patient URL Configuration."""

from django.urls import path

from . import views

app_name = "relatedpersons"

urlpatterns = [
    path("relatedpersons/", views.RelatedPersonListView.as_view()),
    path("relatedpersons/<int:pk>/", views.RelatedPersonDetailView.as_view()),
]



"""Devicedispenses models."""

from django.db import models

from dfhir.base.models import (
    Annotation,
    BaseReference,
    CodeableConcept,
    Identifier,
    Reference,
    SimpleQuantity,
    TimeStampedModel,
)

from . import choices


class DeviceDispenseReceiverReference(BaseReference):
    """DeviceDispenseReceiverReference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="device_dispense_receiver_reference_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="device_dispense_receiver_reference_patient",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="device_dispense_receiver_reference_practitioner",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        related_name="device_dispense_receiver_reference_related_person",
        null=True,
    )
    location = models.ForeignKey(
        "locations.Location",
        on_delete=models.CASCADE,
        related_name="device_dispense_receiver_reference_location",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="device_dispense_receiver_reference_practitioner_role",
        null=True,
    )


class DeviceDispensePerformerActorReference(BaseReference):
    """DeviceDispensePerformerActorReference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="device_dispense_performer_actor_reference_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="device_dispense_performer_actor_reference_patient",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="device_dispense_performer_actor_reference_practitioner",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.CASCADE,
        related_name="device_dispense_performer_actor_reference_organization",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="device_dispense_performer_actor_reference_practitioner_role",
        null=True,
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.CASCADE,
        related_name="device_dispense_performer_actor_reference_device",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        related_name="device_dispense_performer_actor_reference_device_related_person",
        null=True,
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam",
        on_delete=models.CASCADE,
        related_name="device_dispense_performer_actor_reference_device_care_team",
        null=True,
    )


class DeviceDispensePerformer(TimeStampedModel):
    """DeviceDispensePerformer model."""

    function = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="device_dispense_performer_function",
        null=True,
    )
    actor = models.ForeignKey(
        DeviceDispensePerformerActorReference,
        on_delete=models.CASCADE,
        related_name="device_dispense_performer_actor",
        null=True,
    )


class DeviceDispense(TimeStampedModel):
    """DeviceDispense model."""

    identifier = models.ManyToManyField(
        Identifier,
        related_name="device_dispense_identifier",
        blank=True,
    )
    based_on = models.ManyToManyField(
        "careplans.CarePlanDeviceRequestReference",
        related_name="device_dispense_based_on",
        blank=True,
    )
    part_of = models.ManyToManyField(
        "procedures.ProcedureReference",
        related_name="device_dispense_part_of",
        blank=True,
    )
    status = models.CharField(
        max_length=255, choices=choices.DeviceDispenseStatus.choices, null=True
    )
    status_reason = models.ForeignKey(
        "detectedissues.DetectedIssueCodeableReference",
        on_delete=models.CASCADE,
        related_name="device_dispense_status_reason",
        null=True,
    )
    category = models.ManyToManyField(
        CodeableConcept,
        related_name="device_dispense_category",
        blank=True,
    )
    device = models.ForeignKey(
        "devicedefinitions.DeviceDeviceDefinitionCodeableReference",
        on_delete=models.CASCADE,
        related_name="device_dispense_device",
        null=True,
    )
    subject = models.ForeignKey(
        "patients.PatientPractitionerReference",
        on_delete=models.CASCADE,
        related_name="device_dispense_subject",
        null=True,
    )
    receiver = models.ForeignKey(
        DeviceDispenseReceiverReference,
        on_delete=models.CASCADE,
        related_name="device_dispense_receiver",
        null=True,
    )
    encounter = models.ForeignKey(
        "encounters.EncounterReference",
        on_delete=models.CASCADE,
        related_name="device_dispense_encounter",
        null=True,
    )
    supporting_information = models.ManyToManyField(
        Reference,
        related_name="device_dispense_supporting_information",
        blank=True,
    )
    performer = models.ManyToManyField(
        DeviceDispensePerformer,
        related_name="device_dispense_performer",
        blank=True,
    )
    location = models.ForeignKey(
        "locations.LocationReference",
        on_delete=models.CASCADE,
        related_name="device_dispense_location",
        null=True,
    )
    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="device_dispense_type",
        null=True,
    )
    quantity = models.ForeignKey(
        SimpleQuantity,
        on_delete=models.CASCADE,
        related_name="device_dispense_quantity",
        null=True,
    )

    prepared_date = models.DateTimeField(null=True)
    when_handed_over = models.DateTimeField(null=True)
    destination = models.ForeignKey(
        "locations.LocationReference",
        on_delete=models.CASCADE,
        related_name="device_dispense_destination",
        null=True,
    )
    note = models.ManyToManyField(
        Annotation,
        related_name="device_dispense_note",
        blank=True,
    )

    usage_instruction = models.TextField(null=True)
    event_history = models.ManyToManyField(
        "provenances.ProvenanceReference",
        related_name="device_dispense_event_history",
        blank=True,
    )



"""activity definition urls."""

from django.urls import path

from dfhir.activitydefinitions import views

app_name = "activitydefinitions"

urlpatterns = [
    path("activitydefinitions/", views.ActivityDefinitionListView.as_view()),
    path(
        "activitydefinitions/<int:pk>/",
        views.ActivityDefinitionDetailView.as_view(),
    ),
]



"""Care plan urls."""

from django.urls import path

from . import views

app_name = "careplans"


urlpatterns = [
    path("careplans/", views.CarePlanListView.as_view()),
    path("careplans/<int:pk>/", views.CarePlanDetailView.as_view()),
]



"""Vision prescription urls."""

from django.urls import path

from . import views

app_name = "visionprescriptions"


urlpatterns = [
    path(
        "visionprescriptions/",
        views.VisionPrescriptionListView.as_view(),
        name="visionprescriptions-list",
    ),
    path(
        "visionprescriptions/<int:pk>/",
        views.VisionPrescriptionDetailView.as_view(),
        name="visionprescription-details",
    ),
]



"""medication dispense choices."""

from django.db.models import TextChoices


class MedicationDispenseStatus(TextChoices):
    """medication dispense status choices."""

    PREPARATION = "preparation", "preparation"
    IN_PROGRESS = "in-progress", "in_progress"
    ON_HOLD = "on_hold", "on_hold"
    COMPLETED = "completed", "completed"
    CANCELLED = "cancelled", "cancelled"
    ENTERED_IN_ERROR = "entered-in-error", "entered_in_error"
    STOPPED = "stopped", "stopped"
    DECLINED = "declined", "declined"
    UNKNOWN = "unknown", "unknown"



"""inventory item app."""

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class InventoryitemsConfig(AppConfig):
    """Inventory items app config."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "dfhir.inventoryitems"
    verbose_name = _("Inventory Items")



"""inventoru report views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.inventoryreports.models import InventoryReport
from dfhir.inventoryreports.serializer import InventoryReportSerializer


class InventoryReportListView(APIView):
    """inventory report list view."""

    permission_classes = (AllowAny,)

    @extend_schema(responses={200: InventoryReportSerializer(many=True)})
    def get(self, request):
        """Get inventory reports."""
        inventory_reports = InventoryReport.objects.all()
        serializer = InventoryReportSerializer(inventory_reports, many=True)
        return Response(serializer.data)

    @extend_schema(
        request=InventoryReportSerializer, responses={200: InventoryReportSerializer}
    )
    def post(self, request):
        """Post inventory reports."""
        serializer = InventoryReportSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class InventoryReportDetailView(APIView):
    """inventory report detail view."""

    def get_object(self):
        """Get inventory report detail."""
        try:
            return InventoryReport.objects.get(pk=self.kwargs["pk"])
        except InventoryReport.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: InventoryReportSerializer})
    def get(self, request, pk):
        """Get inventory report detail."""
        inventory_report = self.get_object()
        serializer = InventoryReportSerializer(inventory_report)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={200: InventoryReportSerializer})
    def patch(self, request, pk):
        """Patch inventory report detail."""
        inventory_report = self.get_object()
        serializer = InventoryReportSerializer(
            inventory_report, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={200: None})
    def delete(self, request, pk):
        """Delete inventory report detail."""
        inventory_report = self.get_object()
        inventory_report.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""CoverageEligibilityResponse urls."""

from django.urls import path

from . import views

app_name = "coverageeligibilityresponses"

urlpatterns = [
    path(
        "coverageeligibilityresponses/",
        views.CoverageEligibilityResponseListView.as_view(),
        name="list-create",
    ),
    path(
        "coverageeligibilityresponses/<int:pk>/",
        views.CoverageEligibilityResponseDetailView.as_view(),
        name="detail",
    ),
]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('appointmentresponses', '0007_initial'),
        ('appointments', '0003_initial'),
        ('base', '0005_initial'),
        ('careplans', '0004_initial'),
        ('careteams', '0004_initial'),
        ('claims', '0003_initial'),
        ('contracts', '0003_initial'),
        ('coverageeligibilityrequests', '0002_initial'),
        ('devicerequests', '0004_initial'),
        ('devices', '0001_initial'),
        ('documentreferences', '0003_initial'),
        ('enrollmentrequests', '0003_initial'),
        ('groups', '0001_initial'),
        ('immunizationrecommendations', '0002_initial'),
        ('medicationrequests', '0001_initial'),
        ('nutritionorders', '0001_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0002_initial'),
        ('practitionerroles', '0002_initial'),
        ('practitioners', '0001_initial'),
        ('relatedpersons', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='documentreferenceattesterpartyreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_attester_party_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='documentreferenceattesterpartyreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_attester_party_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='documentreferenceattesterpartyreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_attester_party_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='documentreferenceattester',
            name='party',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_attesters_party', to='documentreferences.documentreferenceattesterpartyreference'),
        ),
        migrations.AddField(
            model_name='documentreferenceauthorreference',
            name='care_team',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_author_reference_care_team', to='careteams.careteam'),
        ),
        migrations.AddField(
            model_name='documentreferenceauthorreference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_author_reference_device', to='devices.device'),
        ),
        migrations.AddField(
            model_name='documentreferenceauthorreference',
            name='group',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_author_reference_group', to='groups.group'),
        ),
        migrations.AddField(
            model_name='documentreferenceauthorreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_author_references_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='documentreferenceauthorreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_author_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='documentreferenceauthorreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_author_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='documentreferenceauthorreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_author_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='documentreferenceauthorreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_author_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='documentreferenceauthorreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_author_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='documentreference',
            name='author',
            field=models.ManyToManyField(blank=True, related_name='document_reference_authors', to='documentreferences.documentreferenceauthorreference'),
        ),
        migrations.AddField(
            model_name='documentreferencebasedonreference',
            name='appointment',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_based_on_reference_appointment', to='appointments.appointment'),
        ),
        migrations.AddField(
            model_name='documentreferencebasedonreference',
            name='appointment_response',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_based_on_reference_appointment_response', to='appointmentresponses.appointmentresponse'),
        ),
        migrations.AddField(
            model_name='documentreferencebasedonreference',
            name='care_plan',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_based_on_reference_care_plan', to='careplans.careplan'),
        ),
        migrations.AddField(
            model_name='documentreferencebasedonreference',
            name='claim',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_based_on_reference_claim', to='claims.claim'),
        ),
        migrations.AddField(
            model_name='documentreferencebasedonreference',
            name='contract',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_based_on_reference_contract', to='contracts.contract'),
        ),
        migrations.AddField(
            model_name='documentreferencebasedonreference',
            name='coverage_eligibility_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_based_on_reference_coverage_eligibility_request', to='coverageeligibilityrequests.coverageeligibilityrequest'),
        ),
        migrations.AddField(
            model_name='documentreferencebasedonreference',
            name='device_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_based_on_reference_device_request', to='devicerequests.devicerequest'),
        ),
        migrations.AddField(
            model_name='documentreferencebasedonreference',
            name='enrollment_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_based_on_reference_enrollment_request', to='enrollmentrequests.enrollmentrequest'),
        ),
        migrations.AddField(
            model_name='documentreferencebasedonreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_based_on_references_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='documentreferencebasedonreference',
            name='immunization_recommendation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_based_on_reference_immunization_recommendation', to='immunizationrecommendations.immunizationrecommendation'),
        ),
        migrations.AddField(
            model_name='documentreferencebasedonreference',
            name='medication_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_based_on_reference_medication_request', to='medicationrequests.medicationrequest'),
        ),
        migrations.AddField(
            model_name='documentreferencebasedonreference',
            name='nutrition_order',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_based_on_reference_nutrition_order', to='nutritionorders.nutritionorder'),
        ),
    ]



"""Specimen definition choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class SpecimenDefinitionStatusChoices(models.TextChoices):
    """Specimen definition status choices."""

    DRAFT = "draft", _("draft")
    ACTIVE = "active", _("active")
    RETIRED = "retired", _("retired")
    UNKNOWN = "unknown", _("unknown")


class SpecimenDefinitionTestedTypePreferenceChoices(models.TextChoices):
    """Specimen definition tested type preference choices."""

    PREFERRED = "preferred", _("preferred")
    ALTERNATE = "alternate", _("alternate")
